// AioPage.cpp : implementation file
//

#include "stdafx.h"
#include "DiiDemo.h"
#include "AioPage.h"
#include "DiiDemoDlg.h"
#include "Dii.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAioPage property page

IMPLEMENT_DYNCREATE(CAioPage, CPropertyPage)

CAioPage::CAioPage() : CPropertyPage(CAioPage::IDD)
{
	//{{AFX_DATA_INIT(CAioPage)
	m_iOutputChannel = 0;
	m_dwOutputValue = 0;
	//}}AFX_DATA_INIT

	m_bIsOutputting = TRUE;
}

CAioPage::~CAioPage()
{
}

void CAioPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAioPage)
	DDX_Control(pDX, IDC_CHANNEL_SPIN, m_channelSpin);
	DDX_Text(pDX, IDC_CHANNEL_NUMBER, m_iOutputChannel);
	DDX_Text(pDX, IDC_OUTPUT_VALUE, m_dwOutputValue);
	//}}AFX_DATA_MAP

	if (!pDX->m_bSaveAndValidate)
	{
		ASSERT (GetParent() && GetParent()->IsKindOf (RUNTIME_CLASS (CDiiDemoDlg)));

		DWORD dwMaxChannels = DiiGetNumberOfAnalogChannels (
								((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
							);

		m_channelSpin.SetRange (0,dwMaxChannels-1);

		m_channelSpin.SetBase (10);

		m_channelSpin.SetPos (0);

		m_bIsOutputting = FALSE;

			//
			// Setup some gain and read the channel range
		OnChangeGain();
	}
}


BEGIN_MESSAGE_MAP(CAioPage, CPropertyPage)
	//{{AFX_MSG_MAP(CAioPage)
	ON_EN_CHANGE(IDC_OUTPUT_VALUE, OnOutputValue)
	ON_WM_TIMER()
	ON_EN_CHANGE(IDC_CHANNEL_NUMBER, OnOutputValue)
	ON_EN_CHANGE(IDC_GAIN, OnChangeGain)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAioPage message handlers


void CAioPage::OnOutputValue() 
{

		//
		// A small hack to prevent reentrancy:
		//
	if (m_bIsOutputting)
		return;

	m_bIsOutputting = TRUE;

		//
		// Aquire and convert the value to output:
		//
	CString strValue;

	GetDlgItem (IDC_OUTPUT_VALUE)->GetWindowText (strValue);

	double dOutValue = atof (strValue);

		//
		// Aquire and convert the channel value:
		//

	GetDlgItem (IDC_CHANNEL_NUMBER)->GetWindowText (strValue);

	DWORD dwChannel = atol (strValue);

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Set the analog output channel, to the value specified by the user:
		//
		// ------------------------------------------------------------------

	ASSERT (GetParent() && GetParent()->IsKindOf (RUNTIME_CLASS (CDiiDemoDlg)));

	if (!DiiSetRealAnalogChannel (	((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
								dwChannel,
								dOutValue
							) )
	{
		strValue = _T("N/A");

		GetDlgItem (IDC_OUTPUT_VALUE)->SetWindowText (strValue);
	}

	m_bIsOutputting = FALSE;
}

BOOL CAioPage::OnKillActive() 
{
	//
	// If this page is no longer active, kill our timer again:
	//

	KillTimer (1);
	
	m_bIsOutputting = TRUE;

	return CPropertyPage::OnKillActive();
}

BOOL CAioPage::OnSetActive() 
{
	//
	// As soon as we're active, start the timer, so we monitor the ports:
	//

	SetTimer (1,1000,NULL);
	
	m_bIsOutputting = FALSE;

	return CPropertyPage::OnSetActive();
}

void CAioPage::OnTimer(UINT nIDEvent) 
{
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Read the input ports and update the fields on screen with it:
		//
		// ------------------------------------------------------------------
	
	ASSERT (GetParent() && GetParent()->IsKindOf (RUNTIME_CLASS (CDiiDemoDlg)));

	CString strValue;

	for (int counter = 0;counter < 8;counter++)
	{
		double dValue = 0.0;

		if (DiiGetRealAnalogChannel (
									((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
									counter,
									&dValue ))
		{
			strValue.Format (_T("%.3f"),dValue);
		} else
		{
			strValue = _T("N/A");
		}

		SetDlgItemText (IDC_CHANNEL_0 + counter,strValue);
	}
	
	CPropertyPage::OnTimer(nIDEvent);
}

void CAioPage::OnChangeGain() 
{
	//
	// The user has changed the gain. Set it up for all channels:
	ASSERT (GetParent() && GetParent()->IsKindOf (RUNTIME_CLASS (CDiiDemoDlg)));

	CString strGain;
	GetDlgItemText (IDC_GAIN,strGain);

	double dGain = atof (strGain);

	for (int iChannel = 0;iChannel < 8;iChannel++)
	{
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Setup the channel gain for the specified channel. In here, we don't
		// worry if the card supports the selected gain or not. If it doesn't
		// it won't have effect on the voltage range
		//
		// ------------------------------------------------------------------

		DiiSetChannelGain (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					TRUE,		// Yes, we setup an input channel
					iChannel,	// specify which channel to setup
					dGain);		// and the new gain

				// the function will return FALSE if unsuccessful.
	}


		//
		// Also display the new channel range (taken from channel 0)

	double dMin = 0.0;
	double dMax = 0.0;

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Ask the device what minimum and maximum values it supports on the
		// given channel. It returns the physical limits, while applying the 
		// prescaler we've setup previously.
		//
		// ------------------------------------------------------------------

	if (DiiGetDeviceChannelRange (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					TRUE,	// Yes, we read it from an input channel
					0,		// the first channel
					&dMin,
					&dMax
				))
	{
		CString strOut;
		strOut.Format ("%.3f - %.3f",dMin,dMax);

		SetDlgItemText (IDC_DEVICE_RANGE,strOut);
	} else
	{
		SetDlgItemText (IDC_DEVICE_RANGE,"N/A");
	}
}
