// AnaBlockIoPage.cpp : implementation file
//

#include "stdafx.h"
#include "DiiDemo.h"
#include "AnaBlockIoPage.h"

#include "DiiDemoDlg.h"
#include "Dii.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

		//
		// This message we'll use for receiving notifications from the
		// DII.
#define MSG_RECEIVED_NOTIFICATION (WM_USER+1)


/////////////////////////////////////////////////////////////////////////////
// CAnaBlockIoPage property page

IMPLEMENT_DYNCREATE(CAnaBlockIoPage, CPropertyPage)

CAnaBlockIoPage::CAnaBlockIoPage() : CPropertyPage(CAnaBlockIoPage::IDD)
{
	//{{AFX_DATA_INIT(CAnaBlockIoPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_bInitialized = FALSE;

	m_dwMaxAnalogValue = 0;

	m_dwCurrentChannel = 0;
}

CAnaBlockIoPage::~CAnaBlockIoPage()
{
}

void CAnaBlockIoPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAnaBlockIoPage)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAnaBlockIoPage, CPropertyPage)
	//{{AFX_MSG_MAP(CAnaBlockIoPage)
	ON_EN_CHANGE(IDC_CHANNEL, OnChangeChannel)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
	ON_MESSAGE (MSG_RECEIVED_NOTIFICATION,OnReceivedNotification)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAnaBlockIoPage message handlers

BOOL CAnaBlockIoPage::OnSetActive() 
	//
	// This function is called when the property page is being made 
	// active. We basically initialize the block I/O interface here, and 
	// intiialize other information needed for the analog interface
{

	do {

				//
				// Find out, at which resolution the device is working, so 
				// we know if the data we're receiving is 16 or 32 bit.
		DWORD dwDeviceResolution = DiiGetResolution (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
				);

		if (!dwDeviceResolution)	// if we don't have a resolution, 
			break;					// this is probably not an analog device

		m_dwMaxAnalogValue = (2 << (dwDeviceResolution-1));

		DWORD dwBufferSize = 
			(STD_FREQUENCY/2) * (m_dwMaxAnalogValue > 0xFFFF ? sizeof (DWORD) : sizeof (WORD));

				//
				// We choose the buffer size such that the DII Driver will send a buffer
				// every 500 ms.... (standard frequency divided by 2)

				//
				// Enable the automatic background block I/O:
				//

		//
		// Setup notification method:
		if (!DiiSetNotificationMethodHwnd (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						m_hWnd,
						MSG_RECEIVED_NOTIFICATION,
						0
					))	break;

		//
		// Request notifications for data blocks being sent or received
		if (!DiiRequestNotification (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					DII_DATA_BLOCK_READ, 
					0 )) break;


		//
		// now initiate block input for all channels:

		DWORD dwInputChannels = 0;
		DWORD dwOutputChannels = 0;
		if (!DiiGetNumberOfAnalogInOutChannels (
							((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
							&dwInputChannels,
							&dwOutputChannels) ||
			!dwInputChannels)
			break;

		for (DWORD dwChannel = 0;dwChannel < dwInputChannels;dwChannel++)
		{
			DiiPrepareBlockInput (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						dwChannel,					// for the first channel only
						DII_TRIGGER_SOFTWARE,	// std. software trigger
						STD_FREQUENCY,			// sample with this frequency (1000 Hz)
						dwBufferSize,			// request appropriate buffer size for the DII!
						NULL					// no direct storage to a file!
					);
		}

		//
		// start the block input:
		if (!DiiStartBlockInput	(
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
				)) break;

		m_bInitialized = TRUE;

	} while (0);

	if (!m_bInitialized)
	{
		GetDlgItem (IDC_BLOCK_IO_DISABLED)->EnableWindow (FALSE);
		GetDlgItem (IDC_CHANNEL)->EnableWindow (FALSE);
	}

	return CPropertyPage::OnSetActive();
}

BOOL CAnaBlockIoPage::OnKillActive() 
{
	if (m_bInitialized)
	{

			//
			// Disable the automatic background block I/O:
			//

		DiiEndBlockInput	(
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
			);

			//
			// Also disable notifications:
			//

		DiiCancelNotification (
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
				0
			);

		m_bInitialized = FALSE;
	}

	
	return CPropertyPage::OnKillActive();
}

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// This message call back is called whenever a notification was
		// received. We basically plot the received data to the screen:
		// 
		// ------------------------------------------------------------------

LONG CAnaBlockIoPage::OnReceivedNotification (WPARAM,LPARAM lParam)
{
		//
		// The LPARAM parameter points to our LPDII_NOFICATION_DATA structure

	LPDII_NOTIFICATION_DATA lpNotifData = LPDII_NOTIFICATION_DATA (lParam);


		//
		// find out if this data is for the channel we're currently displaying, if not, 
		// ignore it: (don't forget to call DiiReleaseNotificationData!)
	if (lpNotifData->dwNotificationParam == m_dwCurrentChannel)
	{
		//
		// clear the wave display:

		CRect waveRect;
		GetDlgItem (IDC_WAVE_FRAME)->GetClientRect (waveRect);

		CDC * pDC = GetDlgItem (IDC_WAVE_FRAME)->GetDC();
		pDC->FillSolidRect (waveRect,RGB (0xFF,0xFF,0xFF));

		CPen blackPen;
		blackPen.FromHandle (HPEN (GetStockObject (BLACK_PEN)));

		//
		// Find out how many items are inside the buffer:
		DWORD dwNumberOfItemsInBuffer = 
			lpNotifData->dwBufferSize / (m_dwMaxAnalogValue < 0xFFFF ? sizeof (DWORD) : sizeof (WORD));

		//
		// Initialize the appropriate pointer into our data structure:
		LPWORD lpwLowResData = m_dwMaxAnalogValue > 0xFFFF ? NULL : LPWORD (lpNotifData->lpBuffer);
		LPDWORD lpdwHighResData = m_dwMaxAnalogValue > 0xFFFF ? LPDWORD (lpNotifData->lpBuffer) : NULL;

		//
		// Initialize other data for painting the wave form:
		int iXPointer = waveRect.left;
		CPen * pOldPen = pDC->SelectObject (&blackPen);
		int iHeight = waveRect.Height();

		while (	iXPointer <= waveRect.right &&
				dwNumberOfItemsInBuffer)
		{
			DWORD dwDataSample = 0;
					// load in the data sample (depending on the resolution)
			if (lpwLowResData)
				dwDataSample = DWORD (*lpwLowResData++);
			else
				dwDataSample = *lpdwHighResData++;


					// find the relative Y axis position for this sample:

			int iYPointer = 0;
			
			if (dwDataSample < m_dwMaxAnalogValue)
				iYPointer = int ((double (dwDataSample) / double (m_dwMaxAnalogValue)) * iHeight);
					//
					// Move or paint to the position, (depending if this is the first sample)
					// (should be optimized)
			if (iXPointer == waveRect.left)
				pDC->MoveTo (iXPointer,waveRect.bottom - iYPointer);
			else 
				pDC->LineTo (iXPointer,waveRect.bottom - iYPointer);
			
			iXPointer++;
			dwNumberOfItemsInBuffer--;
		}

		pDC->SelectObject (pOldPen);

		ReleaseDC (pDC);
	}

		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// We MUST release the notification data after processing it, so the
		// DII can automatically reuse buffers!
		// 
		// ------------------------------------------------------------------

	DiiReleaseNotificationData (
					((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					lpNotifData
				);
	return 1;
}

void CAnaBlockIoPage::OnChangeChannel() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CPropertyPage::OnInitDialog()
	// function to send the EM_SETEVENTMASK message to the control
	// with the ENM_CHANGE flag ORed into the lParam mask.
	
	// TODO: Add your control notification handler code here

	m_dwCurrentChannel = GetDlgItemInt (IDC_CHANNEL);
}


void CAnaBlockIoPage::OnDestroy() 
{
	//
	// If the window is destroyed before the block I/O interface is stopped, stop it now


	if (m_bInitialized)
	{

			//
			// Disable the automatic background block I/O:
			//

		DiiEndBlockInput	(
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle()
			);

			//
			// Also disable notifications:
			//

		DiiCancelNotification (
				((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
				0
			);

		m_bInitialized = FALSE;
	}

	CPropertyPage::OnDestroy();
	
	
}
