// DioPage.cpp : implementation file
//

#include "stdafx.h"
#include "DiiDemo.h"
#include "DioPage.h"
#include "DiiDemoDlg.h"

#include "Dii.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDioPage property page

#define WM_DIOUPDATE_STATUS (WM_USER+1)

IMPLEMENT_DYNCREATE(CDioPage, CPropertyPage)

CDioPage::CDioPage() : CPropertyPage(CDioPage::IDD)
{
	//{{AFX_DATA_INIT(CDioPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_bIsAutoTestMode = FALSE;
	m_wAutoLastPortMask = 0;
	m_dwInputBank = m_dwOutputBank = 0;
}

CDioPage::~CDioPage()
{
}

void CDioPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDioPage)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDioPage, CPropertyPage)
	//{{AFX_MSG_MAP(CDioPage)
	ON_BN_CLICKED(IDC_AUTO_TEST, OnAutoTest)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_USE_INTERRUPTS, OnUseInterrupts)
	ON_EN_CHANGE(IDC_INPUT_BANK, OnChangeBank)
	ON_EN_CHANGE(IDC_OUTPUT_BANK, OnChangeBank)
	//}}AFX_MSG_MAP

	ON_COMMAND_RANGE(IDC_OUTP_1,IDC_OUTP_1+15,OnBitToggled)
	ON_MESSAGE(WM_DIOUPDATE_STATUS,OnUpdateStatus)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDioPage message handlers

void CDioPage::OnAutoTest() 
{
		//
		// Toggle the auto test mode:
		//

	m_bIsAutoTestMode = !m_bIsAutoTestMode;
}

BOOL CDioPage::OnKillActive() 
{
	//
	// If this page is no longer active, kill our timer again:
	//

	KillTimer (1);

	//
	// Also stop any notifications from the device (if any are enabled)
	//

	if (IsDlgButtonChecked (IDC_USE_INTERRUPTS))
		DiiCancelNotification (((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),0);
	
	return CPropertyPage::OnKillActive();
}

BOOL CDioPage::OnSetActive() 
{
		//
		// Due to the fact that many users didn't understand the handling of
		// the 8255 card, we now add some intelligence: If we have an 8255 card, 
		// we automatically disable the second ports (so we only have 8-bit), and
		// set the input bank accordingly:
	if (DiiSet8255Config (((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
					0,
					0x00))
	{
		for (int iCounter = 0;iCounter < 8;iCounter++)
			GetDlgItem (IDC_OUTP_9+iCounter)->EnableWindow (FALSE);

		SetDlgItemInt (IDC_OUTPUT_BANK,0);
		SetDlgItemInt (IDC_INPUT_BANK,1);

		OnChangeBank();

	}

	//
	// As soon as we're active, start the timer, so we monitor the ports:
	// (we use a long interval, so the effect of notification based
	// processing becomes evident)
	//

	SetTimer (1,1500,NULL);

	//
	// Call our function to enable/disable notifications
	//

	OnUseInterrupts();
	
	return CPropertyPage::OnSetActive();
}

void CDioPage::OnBitToggled (UINT idButton)
	//
	// This function is called, whenever the user has clicked on one
	// of the output line buttons
	//
{
	
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Set the new state of the ports.
		// We have to get the device handle from our parent, which is the
		// property sheet. Then set the state of each output bank:
		//
		// ------------------------------------------------------------------

	BOOL bState = IsDlgButtonChecked (idButton);
	DWORD dwLine = idButton - IDC_OUTP_1;

		//
		// Now take into account any output bank configured:
	dwLine += m_dwOutputBank * 8;

	ASSERT (GetParent() && GetParent()->IsKindOf (RUNTIME_CLASS (CDiiDemoDlg)));

	if (!DiiSetDigitalBit (	((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
							dwLine,
							bState))
	{
		//
		// If unsuccessfull, restore the state of the output button:
		//

		CheckDlgButton (idButton,FALSE);
	}
}

void CDioPage::OnTimer(UINT nIDEvent) 
{
		//
		// First determine, wether we've to update the output ports:
		//
	
	ASSERT (GetParent() && GetParent()->IsKindOf (RUNTIME_CLASS (CDiiDemoDlg)));
	
	if (m_bIsAutoTestMode)
	{
		m_wAutoLastPortMask <<= 1;

		if (!m_wAutoLastPortMask)
			m_wAutoLastPortMask = 1;


		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Set the new state of the ports.
		// We have to get the device handle from our parent, which is the
		// property sheet. Then set the state of each output bank:
		//
		// ------------------------------------------------------------------
	

		if (!DiiSetDigitalByte(	((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
								m_dwOutputBank,
								BYTE (m_wAutoLastPortMask)))
		{
			//
			// If unsuccessful, remove the lower portion of the port mask:
			//
			m_wAutoLastPortMask &= 0xFF00;	
		}

		if (!DiiSetDigitalByte(	((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
								m_dwOutputBank+1,
								m_wAutoLastPortMask >> 8))
		{
			//
			// If unsuccessful, remove the higher portion of the port mask:
			//
			m_wAutoLastPortMask &= 0x00FF;	
		}


			//
			// Update the output buttons with the new state:
			//

		WORD wMask = 1;
		for (int counter = 0;counter < 16;counter++)
		{
			CheckDlgButton (IDC_OUTP_1 + counter,BOOL ((m_wAutoLastPortMask & wMask) != 0));

			wMask <<= 1;
		}


	}

		//
		// Call the message handler to update the status of the output ports:

	OnUpdateStatus();

	CPropertyPage::OnTimer(nIDEvent);
}

void CDioPage::OnUpdateStatus(WPARAM,LPARAM)
	//
	// This function is called to update the status. It can be called
	// from the timer routine or from the background routine that received an interrupt
	// notification.
	//
{
		// ------------------------------------------------------------------
		//                        DII Library Calls:
		//
		// Our Task here:
		// Read the input ports and update the buttons on screen with it:
		//
		// ------------------------------------------------------------------

	WORD wPortMask = 0;

	BYTE byPortState;

	if (DiiGetDigitalByte (	((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
							m_dwInputBank+1,
							&byPortState) )
	{
		wPortMask = byPortState;

		wPortMask <<= 8;
	}

	if (DiiGetDigitalByte (	((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
							m_dwInputBank,
							&byPortState) )
	{
		wPortMask |= byPortState;
	}

		//
		// Update the output buttons with the new state:
		//

	WORD wMask = 1;
	for (int counter = 0;counter < 16;counter++)
	{
		CheckDlgButton (IDC_INP_1 + counter,BOOL ((wPortMask & wMask) != 0));

		wMask <<= 1;
	}

	
}

void CDioPage::OnUseInterrupts() 
{
	//
	// The user has either selected to use or not use interrupts from
	// the card. We try to setup notification handling with the device
	//

	// ------------------------------------------------------------------
	//                        DII Library Calls:
	//
	// Our Task here:
	// Enable notification from the device. We're interested in receiving
	// DII_SIGNAL_CHANGE notifications, which are sent whenenver an input
	// signal has changed.
	// 
	// Since we're accessing the DLL version of the DII, we have two choices:
	// Either a callback function, which is called upon a notification, 
	// or a Windows message. 
	//
	// We use the callback mechanism here.
	//
	// ------------------------------------------------------------------
	
	if (IsDlgButtonChecked (IDC_USE_INTERRUPTS))
	{	
			// 
			// Register the Callback routine with the DII:
			//
		if (!DiiSetNotificationMethodCallBack (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						ReceivedNotification))
		{
			AfxMessageBox ("Unable to register the Callback routine with the DII.\nProbably, the current device does not support notifications.");

			CheckDlgButton (IDC_USE_INTERRUPTS,FALSE);

			return;
		}

			//
			// Now start requesting notifications:
			// (Note that we have a pointer to our object passed back into
			// the callback routine, through the DII:)
			//
		if (!DiiRequestNotification (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						DII_SIGNAL_CHANGED,
						DWORD (this))
					)
		
		{
			AfxMessageBox ("Unable to request notifications from the DII.\nProbably, the current device does not support notifications,\nor the IRQ cannot be used in your computer.");

			CheckDlgButton (IDC_USE_INTERRUPTS,FALSE);

			return;
		}
		
	} else
	{
			//
			// It's possible that we call DiiCancelNotifications too often,
			// but the DII is protected against this.
			//
		DiiCancelNotification (((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),0);
	}
}


void CDioPage::ReceivedNotification (LPDII_NOTIFICATION_DATA lpNotificationData)
{
	//
	// This function is called whenever signals have changed. We post
	// a WM_DIOUPDATE_STATUS message to our window, causing it to update the status
	// of all input ports.
	//
	// This will have the effect that we almost immediately update the display
	// whenever the output ports have changed.
	//
	//
	// Note: We could have also requested a windows message notification, instead. 
	// This would have prevented writing this function, because the DII automatically
	// posts the windows message.
	//
	// As an example, you can change the DiiSetNotification... function call above, to
	// DiiSetNotificationMethodHwnd.
	// 

	((CDioPage *)lpNotificationData->dwUserData)->PostMessage (
							WM_DIOUPDATE_STATUS,
							0,
							0
						);
}

void CDioPage::OnChangeBank() 
{
	//
	// This method is called, whenever the user decided to change the bank
	// We first gather the values from the user interface, then we
	// attempt to perform an 8255 port setup, in case we're talking to
	// an 8255 card.

	m_dwInputBank = DWORD (GetDlgItemInt (IDC_INPUT_BANK,NULL,FALSE));
	m_dwOutputBank = DWORD (GetDlgItemInt (IDC_OUTPUT_BANK,NULL,FALSE));

		//
		// Now perform the setup of the 8255 chip. Note that the ports
		// may be on the same chip, so we have to combine the setup:
	DWORD dwInputChipIndex = m_dwInputBank / 3;
	DWORD dwOutputChipIndex = m_dwOutputBank / 3;

		// for our convenience, we set the other (unused) ports to output.
	BYTE byConfig = 0;		// a setup of '0' means, all channels output.

	DiiSet8255Config (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						dwOutputChipIndex,
						byConfig
					);

	switch (m_dwInputBank % 3)
	{
		case 0:
			byConfig |= 0x10;
			break;
		case 1:
			byConfig |= 0x2;
			break;
		case 2:
			byConfig |= 0x9;
			break;
	}

	DiiSet8255Config (
						((CDiiDemoDlg *)GetParent())->GetDeviceHandle(),
						dwInputChipIndex,
						byConfig
					);

}


